==> cramr_ui_lib-0.0.12.tar.gz: lib/widgets/text_checker.dart <==
import 'package:cramr_ui_lib/model/verb_test_model.dart';
import 'package:cramr_ui_lib/widgets/pronoun_field.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:provider/provider.dart';

/// This widget checks whether the typed word corresponds to the expected one.
/// Initially, the checkmark is grey. As long as the user types the right
/// characters, the checkmark remains orange. When wrong, the checkmark becomes
/// red. Finally, when the word is completely correct, the checkmark becomes
/// green. Due to the coupling with the Provider framework and the
/// [VerbTestModel] we need the widget to be initialised with the person of the
/// indicative under evaluation.
class TextChecker extends StatefulWidget {
  const TextChecker({Key? key, required this.currentPerson}) : super(key: key);

  final int currentPerson;

  @override
  State<TextChecker> createState() => TextCheckerState();
}

/// The state consists of the Icon (and its color), the comparison status and
/// the word that is actually typed.
class TextCheckerState extends State<TextChecker> {
  late Icon statusIcon = _updatedIcon();
  ComparisonStatus _comparisonStatus = ComparisonStatus.notStarted;
  String actual = '';

  final TextEditingController _actualText = TextEditingController();

  @override
  Widget build(BuildContext context) {
    return Card(
      child: Row(
        children: [
          PronounField(currentPerson: widget.currentPerson),
          _actualTextField(context, widget.currentPerson),
          _updatedIcon(),
        ],
      ),
      elevation: 20,
    );
  }

  Widget _actualTextField(BuildContext context, int person) {
    return Consumer<VerbTestModel>(builder: (context, testModel, child) {
      return SizedBox(
        height: 40,
        width: 170,
        child: TextField(
          key: const Key('actualVerb'),
          obscureText: false,
          autofocus: true,
          decoration: const InputDecoration(
            border: OutlineInputBorder(),
          ),
          readOnly: false,
          controller: _actualText,
          onChanged: (input) {
            actual = _actualText.text;
            _validateInput(context, widget.currentPerson,
                testModel.verb.indicative[person]);
            setState(() {
              statusIcon = _updatedIcon();
            });
          },
        ),
      );
    });
  }

  void _validateInput(BuildContext context, int person, String expected) {
    if (actual.isEmpty) {
      _comparisonStatus = ComparisonStatus.notStarted;
    } else if (actual.length > expected.length) {
      _comparisonStatus = ComparisonStatus.moreThanOneWrong;
    } else if (actual.length == expected.length) {
      if (actual == expected) {
        _comparisonStatus = ComparisonStatus.completelyCorrect;
      }
    } else if (actual.length < expected.length) {
      if (expected.substring(0, actual.length) == actual) {
        _comparisonStatus = ComparisonStatus.stillCorrect;
      } else {
        _comparisonStatus = ComparisonStatus.firstWrong;
      }
    }
  }

  Icon _updatedIcon() {
    return Icon(
      Icons.assignment_turned_in_outlined,
      key: const Key('comparisonStatusIcon'),
      color: _updateIconColor(),
    );
  }

  Color _updateIconColor() {
    switch (_comparisonStatus) {
      case ComparisonStatus.notStarted:
        return Colors.grey;
      case ComparisonStatus.stillCorrect:
        return Colors.orange;
      case ComparisonStatus.completelyCorrect:
        return Colors.green;
      case ComparisonStatus.firstWrong:
        return Colors.red;
      case ComparisonStatus.moreThanOneWrong:
        return Colors.red;
      default:
        return Colors.grey;
    }
  }
}

/// Status of comparison between an expected indicative and what is actually
/// typed.
enum ComparisonStatus {
  notStarted,
  firstWrong,
  moreThanOneWrong,
  stillCorrect,
  completelyCorrect
}

==> cramr_ui_lib-0.0.13.tar.gz: lib/widgets/text_checker.dart <==
import 'package:cramr_ui_lib/model/verb_test_model.dart';
import 'package:cramr_ui_lib/widgets/pronoun_field.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:provider/provider.dart';

/// This widget checks whether the typed word corresponds to the expected one.
/// Initially, the checkmark is grey. As long as the user types the right
/// characters, the checkmark remains orange. When wrong, the checkmark becomes
/// red. Finally, when the word is completely correct, the checkmark becomes
/// green. Due to the coupling with the Provider framework and the
/// [VerbTestModel] we need the widget to be initialised with the person of the
/// indicative under evaluation.
class TextChecker extends StatefulWidget {
  const TextChecker({Key? key, required this.currentPerson}) : super(key: key);

  final int currentPerson;

  @override
  State<TextChecker> createState() => TextCheckerState();
}

/// The state consists of the Icon (and its color), the comparison status and
/// the word that is actually typed.
class TextCheckerState extends State<TextChecker> {
  late Icon statusIcon = _updatedIcon();
  ComparisonStatus _comparisonStatus = ComparisonStatus.notStarted;
  String actual = '';

  final TextEditingController _actualText = TextEditingController(text: '');


  @override
  void initState() {
    super.initState();
    _actualText.text = '';
  }

  @override
  Widget build(BuildContext context) {
    return Card(
      child: Row(
        children: [
          PronounField(currentPerson: widget.currentPerson),
          _actualTextField(context, widget.currentPerson),
          _updatedIcon(),
        ],
      ),
      elevation: 20,
    );
  }

  Widget _actualTextField(BuildContext context, int person) {
    return Consumer<VerbTestModel>(builder: (context, testModel, child) {
      return SizedBox(
        height: 40,
        width: 170,
        child: TextField(
          key: Key('actualVerb_$person'),
          obscureText: false,
          autofocus: true,
          decoration: const InputDecoration(
            border: OutlineInputBorder(),
          ),
          readOnly: false,
          controller: _actualText,
          onChanged: (input) {
            actual = _actualText.text;
            _validateInput(context, widget.currentPerson,
                testModel.verb.indicative[person]);
            setState(() {
              statusIcon = _updatedIcon();
            });
          },
        ),
      );
    });
  }

  void _validateInput(BuildContext context, int person, String expected) {
    if (actual.isEmpty) {
      _comparisonStatus = ComparisonStatus.notStarted;
    } else if (actual.length > expected.length) {
      _comparisonStatus = ComparisonStatus.moreThanOneWrong;
    } else if (actual.length == expected.length) {
      if (actual == expected) {
        _comparisonStatus = ComparisonStatus.completelyCorrect;
      }
    } else if (actual.length < expected.length) {
      if (expected.substring(0, actual.length) == actual) {
        _comparisonStatus = ComparisonStatus.stillCorrect;
      } else {
        _comparisonStatus = ComparisonStatus.firstWrong;
      }
    }
  }

  Icon _updatedIcon() {
    return Icon(
      Icons.assignment_turned_in_outlined,
      key: const Key('comparisonStatusIcon'),
      color: _updateIconColor(),
    );
  }

  Color _updateIconColor() {
    switch (_comparisonStatus) {
      case ComparisonStatus.notStarted:
        return Colors.grey;
      case ComparisonStatus.stillCorrect:
        return Colors.orange;
      case ComparisonStatus.completelyCorrect:
        return Colors.green;
      case ComparisonStatus.firstWrong:
        return Colors.red;
      case ComparisonStatus.moreThanOneWrong:
        return Colors.red;
      default:
        return Colors.grey;
    }
  }
}

/// Status of comparison between an expected indicative and what is actually
/// typed.
enum ComparisonStatus {
  notStarted,
  firstWrong,
  moreThanOneWrong,
  stillCorrect,
  completelyCorrect
}
